// Copyright (c) 2022 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "DawnPokerGame.hpp"

using namespace Dawn;

std::shared_ptr<SceneItem> cubeCreate(std::shared_ptr<Scene> scene) {
  auto cubeObject = scene->createSceneItem();
  auto cubeMeshRenderer = cubeObject->addComponent<MeshRenderer>();
  auto cubeMaterial = cubeObject->addComponent<Material>();
  cubeMeshRenderer->mesh = std::make_shared<Mesh>();
  cubeMeshRenderer->mesh->createBuffers(CUBE_VERTICE_COUNT, CUBE_INDICE_COUNT);
  CubeMesh::buffer( 
    *cubeMeshRenderer->mesh,
    glm::vec3(-0.5f, -0.5f, -0.5f), glm::vec3(1, 1, 1),
    0, 0
  );
  return cubeObject;
}

std::shared_ptr<SceneItem> cubeParent;

DawnGame::DawnGame(DawnHost &host) :
  host(host),
  renderManager(*this)
{
}

int32_t DawnGame::init() {
  this->assetManager.init();
  this->renderManager.init();

  this->scene = std::make_shared<Scene>(*this);

  auto cameraObject = this->scene->createSceneItem();
  auto camera = cameraObject->addComponent<Camera>();
  camera->transform.lookAt(glm::vec3(5, 5, 5), glm::vec3(0, 0, 0));
  
  cubeParent = cubeCreate(this->scene);
  auto cubeChild = cubeCreate(this->scene);
  cubeChild->getComponent<Material>()->colorValues[cubeChild->getComponent<Material>()->getShader()->getParameterByName("u_Color")] = COLOR_MAGENTA;
  cubeChild->transform.setParent(&cubeParent->transform);
  cubeChild->transform.setLocalPosition(glm::vec3(1, 0, 0));
  cubeParent->transform.setLocalPosition(glm::vec3(-0.5f, 0, 0));

  return DAWN_GAME_INIT_RESULT_SUCCESS;
}

int32_t DawnGame::update(float_t delta) {
  this->assetManager.update();

  if(this->scene != nullptr) this->scene->update();
  
  cubeParent->transform.setLocalPosition(cubeParent->transform.getLocalPosition() + glm::vec3(-delta, 0, 0));

  this->renderManager.update();

  return DAWN_GAME_UPDATE_RESULT_SUCCESS;
}

DawnGame::~DawnGame() {
  
}